@@ -1,5 +1,5 @@
 use crate::config::Context;
-use crate::{util, AppId, Verbs};
+use crate::{trust, util, AppId, Verbs};
 use anyhow::{anyhow, Context as AnyhowContext, Result};
 use oauth2::TokenResponse;
 use reqwest::blocking::{Client, Response};
@@ -13,7 +13,7 @@ fn craft_url(base: &Url, app_id: Option<&str>) -> String {
         Some(app) => format!("/{}", app),
         None => String::new(),
     };
-    format!("{}{}/apps{}", base, util::API_PATH, app)
+    format!("{}{}/apps{}", base, util::REGISTRY_API_PATH, app)
 }
 
 pub fn create(
@@ -50,7 +50,7 @@ pub fn read(config: &Context, app: AppId) -> Result<()> {
     get(config, &app).map(|res| util::print_result(res, app.to_string(), Verbs::get))
 }
 
-pub fn delete(config: &Context, app: AppId) -> Result<()> {
+pub fn delete(config: &Context, app: AppId, ignore_missing: bool) -> Result<()> {
     let client = Client::new();
     let url = craft_url(&config.registry_url, Some(&app));
 
@@ -59,7 +59,13 @@ pub fn delete(config: &Context, app: AppId) -> Result<()> {
         .bearer_auth(&config.token.access_token().secret())
         .send()
         .context("Can't get app.")
-        .map(|res| util::print_result(res, format!("App {}", &app), Verbs::delete))
+        .map(|res| {
+            if ignore_missing && res.status() == StatusCode::NOT_FOUND {
+                exit(0);
+            } else {
+                util::print_result(res, format!("App {}", &app), Verbs::delete)
+            }
+        })
 }
 
 pub fn edit(config: &Context, app: AppId, file: Option<&str>) -> Result<()> {
@@ -136,6 +142,67 @@ fn get(config: &Context, app: &str) -> Result<Response> {
         .context("Can't retrieve app data.")
 }
 
+pub fn add_trust_anchor(
+    config: &Context,
+    app: &str,
+    keyout: Option<&str>,
+    key_pair_algorithm: Option<trust::SignAlgo>,
+    days: Option<&str>,
+    key_input: Option<rcgen::KeyPair>,
+) -> Result<()> {
+    let res = get(config, &app);
+    match res {
+        Ok(r) => match r.status() {
+            StatusCode::OK => {
+                let app_obj = r.text().unwrap_or_else(|_| "{}".to_string());
+                let mut app_obj: Value = serde_json::from_str(&app_obj)?;
+                app_obj["spec"]["trustAnchors"] =
+                    trust::create_trust_anchor(app, keyout, key_pair_algorithm, days, key_input)?;
+
+                put(config, app, app_obj)
+                    .map(|p| util::print_result(p, format!("App {}", &app), Verbs::edit))
+            }
+            e => {
+                log::error!("Error : could not retrieve app: {}", e);
+                util::exit_with_code(e)
+            }
+        },
+        Err(e) => {
+            log::error!("Error : could not retrieve app: {}", e);
+            exit(2);
+        }
+    }
+}
+
+pub fn get_trust_anchor(config: &Context, app: &str) -> Result<String> {
+    let res = get(config, &app);
+    match res {
+        Ok(r) => match r.status() {
+            StatusCode::OK => {
+                let app_obj = r.text().unwrap_or_else(|_| "{}".to_string());
+                let app_obj_json: Value = serde_json::from_str(&app_obj)?;
+                let cert =
+                    app_obj_json["spec"]["trustAnchors"]["anchors"][0]["certificate"].clone();
+
+                if cert == Value::Null {
+                    log::error!("No trust anchor found in this application.");
+                    exit(1);
+                }
+
+                Ok(cert.to_string().replace("\"", ""))
+            }
+            e => {
+                log::error!("Error : could not retrieve app: {}", e);
+                util::exit_with_code(e)
+            }
+        },
+        Err(e) => {
+            log::error!("Error : could not retrieve app: {}", e);
+            exit(2);
+        }
+    }
+}
+
 fn put(config: &Context, app: &str, data: serde_json::Value) -> Result<Response> {
     let client = Client::new();
     let url = craft_url(&config.registry_url, Some(app));
