@@ -1,9 +1,12 @@
+use crate::trust::SignAlgo;
+
 use anyhow::{anyhow, Context as AnyhowContext, Result};
 use serde::{Deserialize, Serialize};
 use std::{env, fs::create_dir_all, fs::write, fs::File, path::Path, process::exit};
 
 use crate::AppId;
 use chrono::{DateTime, Utc};
+use core::fmt;
 use dirs::config_dir;
 use oauth2::basic::BasicTokenResponse;
 use tabular::{Row, Table};
@@ -18,7 +21,7 @@ pub struct Config {
     //todo : when loading, put a ref to the active context for faster access
     // to avoid looping through the contexts each time.
     // #[serde(skip)]
-    // active_ctx_ref: &Context
+    // active_ctx_ref: Option<&Context>
 }
 
 #[derive(Serialize, Deserialize, Debug, Clone)]
@@ -26,6 +29,7 @@ pub struct Context {
     pub name: ContextId,
     pub drogue_cloud_url: Url,
     pub default_app: Option<AppId>,
+    pub default_algo: Option<String>,
     pub auth_url: Url,
     pub token_url: Url,
     pub registry_url: Url,
@@ -38,6 +42,7 @@ impl Config {
         Config {
             active_context: String::new(),
             contexts: Vec::new(),
+            //            active_ctx_ref: None,
         }
     }
     pub fn from(path: Option<&str>) -> Result<Config> {
@@ -50,6 +55,7 @@ impl Config {
         let config: Config =
             serde_yaml::from_reader(file).context("Invalid configuration file.")?;
 
+        //       config.active_ctx_ref = config.get_active_context().ok();
         Ok(config)
     }
 
@@ -209,11 +215,16 @@ impl Config {
             Err(anyhow!("Context {} does not exist in config file.", name))
         }
     }
+}
 
-    //todo implement a display Trait ?
-    pub fn show(&self) -> Result<()> {
-        println!("{}", serde_yaml::to_string(&self)?);
-        Ok(())
+impl fmt::Display for Config {
+    fn fmt(&self, f: &mut fmt::Formatter) -> fmt::Result {
+        write!(
+            f,
+            "{}",
+            serde_yaml::to_string(&self)
+                .unwrap_or("Cannot deserialize the config. The file may be corrupted.".to_string())
+        )
     }
 }
 
@@ -225,6 +236,10 @@ impl Context {
     pub fn set_default_app(&mut self, app: AppId) {
         self.default_app = Some(app);
     }
+
+    pub fn set_default_algo(&mut self, algo: SignAlgo) {
+        self.default_algo = Some(algo.as_ref().to_string())
+    }
 }
 
 // use the provided config path or `$DRGCFG` value if set
