@@ -1,6 +1,7 @@
 use crate::config::Context;
 use crate::{util, AppId, DeviceId, Verbs};
 use anyhow::{anyhow, Context as AnyhowContext, Result};
+use json_value_merge::Merge;
 use oauth2::TokenResponse;
 use reqwest::blocking::Client;
 use reqwest::blocking::Response;
@@ -17,13 +18,18 @@ fn craft_url(base: &Url, app_id: &str, device_id: Option<&str>) -> String {
     format!(
         "{}{}/apps/{}/devices{}",
         base,
-        util::API_PATH,
+        util::REGISTRY_API_PATH,
         app_id,
         device
     )
 }
 
-pub fn delete(config: &Context, app: AppId, device_id: DeviceId) -> Result<()> {
+pub fn delete(
+    config: &Context,
+    app: AppId,
+    device_id: DeviceId,
+    ignore_missing: bool,
+) -> Result<()> {
     let client = Client::new();
     let url = craft_url(&config.registry_url, &app, Some(&device_id));
 
@@ -32,7 +38,13 @@ pub fn delete(config: &Context, app: AppId, device_id: DeviceId) -> Result<()> {
         .bearer_auth(&config.token.access_token().secret())
         .send()
         .context("Can't delete device.")
-        .map(|res| util::print_result(res, format!("Device {}", device_id), Verbs::delete))
+        .map(|res| {
+            if ignore_missing && res.status() == StatusCode::NOT_FOUND {
+                exit(0);
+            } else {
+                util::print_result(res, format!("Device {}", device_id), Verbs::delete)
+            }
+        })
 }
 
 pub fn read(config: &Context, app: AppId, device_id: DeviceId) -> Result<()> {
@@ -47,6 +59,12 @@ pub fn create(
     app_id: AppId,
     file: Option<&str>,
 ) -> Result<()> {
+    let data = if data == json!({}) {
+        json!({"credentials": {}})
+    } else {
+        data
+    };
+
     let body = match file {
         Some(f) => util::get_data_from_file(f)?,
         None => {
@@ -132,6 +150,88 @@ pub fn list(config: &Context, app: AppId, labels: Option<String>) -> Result<()>
         Err(anyhow!("Error while requesting devices list."))
     }
 }
+
+pub fn set_gateway(
+    config: &Context,
+    app: AppId,
+    device_id: DeviceId,
+    gateway_id: DeviceId,
+) -> Result<()> {
+    // prepare json data to merge
+    let data = json!({"spec": {
+    "gatewaySelector": {
+        "matchNames": [gateway_id]
+    }
+    }});
+
+    set(config, app, device_id, data)
+}
+
+pub fn set_password(
+    config: &Context,
+    app: AppId,
+    device_id: DeviceId,
+    password: String,
+    username: Option<&str>,
+) -> Result<()> {
+    let credential = match username {
+        Some(user) => json!({"user": {"username": user, "password": password}}),
+        None => json!({ "pass": password }),
+    };
+
+    // prepare json data to merge
+    let data = json!({"spec": {
+    "credentials": {
+        "credentials": [
+          credential
+        ]
+    }
+    }});
+
+    set(config, app, device_id, data)
+}
+
+pub fn add_alias(
+    config: &Context,
+    app: AppId,
+    device_id: DeviceId,
+    new_alias: String,
+) -> Result<()> {
+    // prepare json data to merge
+    let data = json!({"spec": {
+    "alias": [
+          new_alias
+        ]
+    }});
+
+    set(config, app, device_id, data)
+}
+
+// The "set" operation merges the data with what already exists on the server side
+fn set(config: &Context, app: AppId, device_id: DeviceId, data: Value) -> Result<()> {
+    //read device data
+    let res = get(&config, &app, &device_id);
+    match res {
+        Ok(r) => match r.status() {
+            StatusCode::OK => {
+                let mut body: Value =
+                    serde_json::from_str(r.text().unwrap_or_else(|_| "{}".to_string()).as_str())?;
+                body.merge(data);
+                put(&config, &app, &device_id, body)
+                    .map(|p| util::print_result(p, format!("Device {}", device_id), Verbs::edit))
+            }
+            e => {
+                log::error!("Error : could not retrieve device: {}", e);
+                util::exit_with_code(e)
+            }
+        },
+        Err(e) => {
+            log::error!("Error : could not execute request: {}", e);
+            exit(2)
+        }
+    }
+}
+
 fn get(config: &Context, app: &str, device_id: &DeviceId) -> Result<Response> {
     let client = Client::new();
     let url = craft_url(&config.registry_url, app, Some(&device_id));
