@@ -27,7 +27,7 @@ pub fn login(
 ) -> Result<Context> {
     log::info!("Starting authentication process with {}", api_endpoint);
 
-    let (sso_url, registry_url) = util::get_drogue_services_endpoint(api_endpoint.clone())?;
+    let (sso_url, registry_url) = util::get_drogue_services_endpoints(api_endpoint.clone())?;
     let (auth_url, token_url) = util::get_auth_and_tokens_endpoints(sso_url)?;
 
     let token = match refresh_token_val {
@@ -48,6 +48,7 @@ pub fn login(
         name: context_name,
         drogue_cloud_url: api_endpoint,
         default_app: None,
+        default_algo: None,
         token,
         token_url,
         auth_url,
@@ -103,9 +104,6 @@ fn get_token(auth_url: Url, token_url: Url) -> Result<BasicTokenResponse> {
     let state = querry.get("state").unwrap();
     let code = querry.get("code").unwrap();
 
-    let _ = request.respond(Response::from_string(
-        "Authentication code retrieved. This browser can be closed.",
-    ));
     log::info!("Authentication code retrieved.");
     log::debug!("Trading auth code with token using url : {}", token_url);
 
@@ -119,8 +117,19 @@ fn get_token(auth_url: Url, token_url: Url) -> Result<BasicTokenResponse> {
         .set_pkce_verifier(pkce_verifier)
         .request(http_client);
 
+    let browser_msg = match token_result {
+        Ok(_) => "Authentication success. This browser can be closed.",
+        Err(_) => "Authentication failed! This browser can be closed.",
+    };
+    let _ = request.respond(Response::from_string(browser_msg));
+
     // Unwrapping token_result will either produce a Token or a RequestTokenError.
-    token_result.map_err(|_| Error::msg("error retrieving the authentication token"))
+    token_result.map_err(|e| {
+        Error::msg(format!(
+            "error while requesting a token: \n{}",
+            e.to_string()
+        ))
+    })
 }
 
 pub fn verify_token_validity(context: &mut Context) -> Result<bool> {
@@ -175,7 +184,10 @@ fn exchange_token(
     client
         .exchange_refresh_token(refresh_token_val)
         .request(http_client)
-        .map_err(|_| Error::msg("Invalid refresh token"))
+        .map_err(|e| {
+            log::warn!("{:?}", e);
+            Error::msg(format!("While refreshing token : {}", e))
+        })
 }
 
 fn calculate_token_expiration_date(token: &BasicTokenResponse) -> Result<DateTime<Utc>> {
