@@ -1,24 +1,29 @@
-use crate::config::Config;
+use crate::config::{Config, Context};
+use crate::Other_flags;
 use crate::Verbs;
-use anyhow::{Context as AnyhowContext, Result};
+use anyhow::{anyhow, Context as AnyhowContext, Result};
 use chrono::{Duration, Utc};
 use clap::crate_version;
 use clap::ArgMatches;
 use colored_json::write_colored_json;
 use log::LevelFilter;
+use oauth2::TokenResponse;
 use reqwest::blocking::{Client, Response};
 use reqwest::StatusCode;
+use serde_json::Value::String as serde_string;
 use serde_json::{from_str, Value};
 use std::fs;
 use std::io::stdout;
 use std::io::{Read, Write};
 use std::process::exit;
+use tabular::{Row, Table};
 use tempfile::Builder;
 use url::Url;
 
 pub const VERSION: &str = crate_version!();
 pub const COMPATIBLE_DROGUE_VERSION: &str = "0.5.0";
-pub const API_PATH: &str = "api/registry/v1alpha1";
+pub const REGISTRY_API_PATH: &str = "api/registry/v1alpha1";
+pub const COMMAND_API_PATH: &str = "api/command/v1alpha1";
 
 pub fn print_result(r: Response, resource_name: String, op: Verbs) {
     match op {
@@ -34,19 +39,22 @@ pub fn print_result(r: Response, resource_name: String, op: Verbs) {
             StatusCode::OK => show_json(r.text().expect("Empty response")),
             r => exit_with_code(r),
         },
-        Verbs::edit => match r.status() {
+        Verbs::edit | Verbs::set => match r.status() {
             StatusCode::NO_CONTENT => println!("{} updated.", resource_name),
             r => exit_with_code(r),
         },
+        //should never happen.
+        Verbs::cmd => {}
     }
 }
 
-fn show_json<S: Into<String>>(payload: S) {
+pub fn show_json<S: Into<String>>(payload: S) {
     let payload = payload.into();
     match serde_json::from_str(&payload) {
         // show as JSON
         Ok(json) => {
             write_colored_json(&json, &mut stdout().lock()).ok();
+            println!();
         }
         // fall back to plain text output
         Err(_) => println!("{}", payload),
@@ -143,7 +151,7 @@ pub fn print_version(config: &Result<Config>) {
 }
 
 // use drogue's well known endpoint to retrieve endpoints.
-pub fn get_drogue_services_endpoint(url: Url) -> Result<(Url, Url)> {
+pub fn get_drogue_services_endpoints(url: Url) -> Result<(Url, Url)> {
     let client = Client::new();
 
     let url = url.join(".well-known/drogue-endpoints")?;
@@ -171,6 +179,28 @@ pub fn get_drogue_services_endpoint(url: Url) -> Result<(Url, Url)> {
     ))
 }
 
+fn get_drogue_endpoints_authenticated(context: &Context) -> Result<Value> {
+    let client = Client::new();
+    let url = format!("{}api/console/v1alpha1/info", &context.registry_url);
+    let res = client
+        .get(url)
+        .bearer_auth(&context.token.access_token().secret())
+        .send()
+        .context("Can't retrieve drogue services details")?;
+
+    res.json()
+        .context("Cannot deserialize drogue endpoints details")
+}
+
+pub fn get_drogue_websocket_endpoint(context: &Context) -> Result<Url> {
+    let endpoints = get_drogue_endpoints_authenticated(context)?;
+    let ws = endpoints["websocket_integration"]["url"]
+        .as_str()
+        .context("No `websocket_integration` service in drogue endpoints list")?;
+
+    url_validation(ws)
+}
+
 // use keycloak's well known endpoint to retrieve endpoints.
 // http://keycloakhost:keycloakport/auth/realms/{realm}/.well-known/openid-configuration
 pub fn get_auth_and_tokens_endpoints(issuer_url: Url) -> Result<(Url, Url)> {
@@ -199,11 +229,20 @@ pub fn get_auth_and_tokens_endpoints(issuer_url: Url) -> Result<(Url, Url)> {
 }
 
 pub fn log_level(matches: &ArgMatches) -> LevelFilter {
-    match matches.occurrences_of("verbose") {
+    match matches.occurrences_of(Other_flags::verbose) {
         0 => LevelFilter::Error,
-        1 => LevelFilter::Warn,
-        2 => LevelFilter::Info,
-        _ => LevelFilter::Debug,
+        1 => {
+            println!("Log level: WARN");
+            LevelFilter::Warn
+        }
+        2 => {
+            println!("Log level: INFO");
+            LevelFilter::Info
+        }
+        _ => {
+            println!("Log level: DEBUG");
+            LevelFilter::Debug
+        }
     }
 }
 
@@ -254,3 +293,50 @@ pub fn age(str_timestamp: &str) -> Result<String> {
         Ok(format!("{}s", age.num_seconds()))
     }
 }
+
+pub fn print_endpoints(context: &Context, service: Option<&str>) -> Result<()> {
+    let endpoints = get_drogue_endpoints_authenticated(context)?;
+    let endpoints = endpoints.as_object().unwrap();
+
+    if let Some(service) = service {
+        let details = endpoints
+            .get(service)
+            .ok_or_else(|| anyhow!("Service not found in endpoints list."))?;
+        let (host, port) = deserialize_endpoint(details);
+
+        println!("{}{}", host.unwrap(), port);
+    } else {
+        let mut table = Table::new("{:<} {:<}");
+        table.add_row(Row::new().with_cell("NAME").with_cell("URL"));
+
+        for (name, details) in endpoints {
+            let (host, port) = deserialize_endpoint(details);
+            host.map(|h| {
+                table.add_row(
+                    Row::new()
+                        .with_cell(name)
+                        .with_cell(format!("{}{}", h, port)),
+                )
+            });
+        }
+        print!("{}", table);
+    }
+
+    Ok(())
+}
+
+fn deserialize_endpoint(details: &Value) -> (Option<String>, String) {
+    let (host, port) = match details {
+        serde_string(s) => (Some(s.clone()), None),
+        Value::Object(v) => (
+            v.get("url")
+                .or_else(|| v.get("host"))
+                .map(|h| h.as_str().unwrap().to_string()),
+            v.get("port").map(|s| s.as_i64().unwrap()),
+        ),
+        _ => (None, None),
+    };
+
+    let port = port.map_or("".to_string(), |p| format!(":{}", p));
+    (host, port)
+}
